from machine import I2C
import servo
import time
i

i2c = I2C(I2C.I2C0, freq=100000, scl=28, sda=29)
pwm = servo.Servos(i2c)

servo_min = 800  # Min pulse length out of 4096
servo_max = 2200  # Max pulse length out of 4096
deg_max = 120 # Max deg
deg_min = 40 # Min deg


#制御ステート定義
stop = 0
fw = 1
cw = 2
ccw = 3
rw = 4
nt = 5
all_up = 6
all_dn = 7
r_up = 8
l_up = 9



#動作速度定義
sslow = 0.1
slow = 0.05
type = 0.02
fast = 0.01
high = 0

#初期
walk_st = stop
speed = high

#各サーボ/初期値　定義
#右脚腿        Right tight     SV-0
RTG = 0
RTG_n = 75
RTG_open = 60
RTG_close = 120
#右脚膝        Right knee      SV-1
RKN = 1
RKN_n = 90
RKN_open = 120
RKN_close = 52
#右脚足首       Right ankle    SV-2
RAK = 2
RAK_n = 57
RAK_open = 50
RAK_close = 115
#右脚つま先      Right toe       SV-3
RTE = 3
RTE_n = 86
RTE_open = 120
RTE_close = 57

#左脚腿        Left tight      SV-4
LTG = 4
LTG_n = 100
LTG_open = 118
LTG_close = 58
#左脚膝        Left knee       SV-5
LKN = 5
LKN_n = 90
LKN_open = 58
LKN_close = 120
#左脚足首       Left ankle      SV-6
LAK = 6
LAK_n = 110
LAK_open = 120
LAK_close = 60
#左脚つま先      Left toe        SV-7
LTE = 7
LTE_n = 88
LTE_open = 58
LTE_close = 120


def deg2pls(deg):
    setdata = int((deg-deg_min) *  (servo_max-servo_min) / (deg_max - deg_min)) + servo_min
    if setdata <= servo_max:
          if setdata >= servo_min:
                        data = setdata
          else:
                print (setdata)
                print ("add or deg value error < min")
                data = servo_min
    else:
        print (setdata)
        print ("deg value error > max")
        data = servo_max
    print(data)
    return data

def set_servo_deg(channel,deg):
    setdata = int((deg-deg_min) *  (servo_max-servo_min) / (deg_max - deg_min)) + servo_min
    print ("setdata")
    print (setdata)
    if setdata <= servo_max:
          if setdata >= servo_min:
                        pwm.position(channel, us=setdata)
                        print(setdata)
          else:
                print (setdata)
                print ("add or deg value error < min")
    else:
        print (setdata)
        print ("deg value error > max")


def sv_init():
    set_servo_deg(RTG, RTG_n)
    set_servo_deg(RKN, RKN_n)
    set_servo_deg(RAK, RAK_n)
    set_servo_deg(RTE, RTE_n)
    set_servo_deg(LTG, LTG_n)
    set_servo_deg(LKN, LKN_n)
    set_servo_deg(LAK, LAK_n)
    set_servo_deg(LTE, LTE_n)


def sv_spuat():
    set_servo_deg(RTG, RTG_open)
    set_servo_deg(LTG, LTG_open)
    set_servo_deg(RKN, RKN_close+25)
    set_servo_deg(LKN, LKN_close-25)
    set_servo_deg(RAK, RAK_close-15)
    set_servo_deg(LAK, LAK_close+10)
    set_servo_deg(RTE, RTE_open)
    set_servo_deg(LTE, LTE_open)

def sv_stretch():
    set_servo_deg(RTG, RTG_close-10)
    set_servo_deg(LTG, LTG_close+10)
    set_servo_deg(RKN, RKN_open)
    set_servo_deg(LKN, LKN_open)
    time.sleep(0.05)
    set_servo_deg(RAK, RAK_open)
    set_servo_deg(LAK, LAK_open)
    time.sleep(0.01)
    set_servo_deg(RTE, RTE_n-5)
    set_servo_deg(LTE, LTE_n+5)


def sv_walk_fw(wait_time):

    #右足上に上げる
    set_servo_deg(RTG, RTG_n)
    set_servo_deg(RKN, RKN_open-40)
    set_servo_deg(RAK, RAK_close-30)
    set_servo_deg(RTE, RTE_n)
    time.sleep(wait_time)

    #右足前へ出す
    set_servo_deg(RTG, RTG_n+5)
    set_servo_deg(RKN, RKN_close+20)
    set_servo_deg(RAK, RAK_close-40)
    set_servo_deg(RTE, RTE_n)
    time.sleep(wait_time)

    #右足下へ下す
    set_servo_deg(RTG, RTG_n+5)
    set_servo_deg(RKN, RKN_close+20)
    set_servo_deg(RAK, RAK_open+5)
    set_servo_deg(RTE, RTE_open-15)
    time.sleep(wait_time)


    #右足後に出す
    set_servo_deg(RTG, RTG_n)
    set_servo_deg(RKN, RKN_open-40)
    set_servo_deg(RAK, RAK_open+5)
    set_servo_deg(RTE, RTE_close+30)
    time.sleep(wait_time)

    #左足上に上げる
    set_servo_deg(LTG, LTG_n)
    set_servo_deg(LKN, LKN_open+40)
    set_servo_deg(LAK, LAK_close+35)
    set_servo_deg(LTE, LTE_n)
    time.sleep(wait_time)

    #左足前へ出す
    set_servo_deg(LTG, LTG_n-8)
    set_servo_deg(LKN, LKN_close-20)
    set_servo_deg(LAK, LAK_close+35)
    set_servo_deg(LTE, LTE_n)
    time.sleep(wait_time)

    #左足下へ下す
    set_servo_deg(LTG, LTG_n-8)
    set_servo_deg(LKN, LKN_close-20)
    set_servo_deg(LAK, LAK_open-10)
    set_servo_deg(LTE, LTE_open+10)
    time.sleep(wait_time)

    #左足後に出す
    set_servo_deg(LTG, LTG_n)
    set_servo_deg(LKN, LKN_open+35)
    set_servo_deg(LAK, LAK_open-10)
    set_servo_deg(LTE, LTE_close-30)
    time.sleep(wait_time)

def sv_walk_rw(wait_time):

    #右足上に上げる
    set_servo_deg(RTG, RTG_n)
    set_servo_deg(RKN, RKN_n)
    set_servo_deg(RAK, RAK_n+20)
    set_servo_deg(RTE, RTE_n)
    time.sleep(wait_time)

    #右足後に出す
    set_servo_deg(RTG, RTG_n-10)
    set_servo_deg(RKN, RKN_n+5)
    set_servo_deg(RAK, RAK_n+5)
    set_servo_deg(RTE, RTE_n-15)
    time.sleep(wait_time)


    #右足下へ下す
    set_servo_deg(RTG, RTG_n-10)
    set_servo_deg(RKN, RKN_n+5)
    set_servo_deg(RAK, RAK_n-10)
    set_servo_deg(RTE, RTE_close+5)
    time.sleep(wait_time)


    #右足前へ出す
    set_servo_deg(RTG, RTG_n)
    set_servo_deg(RKN, RKN_n)
    set_servo_deg(RAK, RAK_n)
    set_servo_deg(RTE, RTE_n)
    time.sleep(wait_time)




    #左足上に上げる
    set_servo_deg(LTG, LTG_n)
    set_servo_deg(LKN, LKN_n)
    set_servo_deg(LAK, LAK_n-20)
    set_servo_deg(LTE, LTE_n)
    time.sleep(wait_time)

    #左足後に出す
    set_servo_deg(LTG, LTG_n+12)
    set_servo_deg(LKN, LKN_n-5)
    set_servo_deg(LAK, LAK_n-5)
    set_servo_deg(LTE, LTE_n+15)
    time.sleep(wait_time)


    #左足下へ下す
    set_servo_deg(LTG, LTG_n+12)
    set_servo_deg(LKN, LKN_n-5)
    set_servo_deg(LAK, LAK_n+5)
    set_servo_deg(LTE, LTE_close-10)
    time.sleep(wait_time)


    #左足前へ出す
    set_servo_deg(LTG, LTG_n)
    set_servo_deg(LKN, LKN_n)
    set_servo_deg(LAK, LAK_n)
    set_servo_deg(LTE, LTE_n)
    time.sleep(wait_time)








def sv_walk_ccw(wait_time):
    #右足上に上げる
    set_servo_deg(RTG, RTG_n)
    set_servo_deg(RKN, RKN_open-40)
    set_servo_deg(RAK, RAK_close-30)
    set_servo_deg(RTE, RTE_n)
    time.sleep(wait_time)

    #右足前へ出す
    set_servo_deg(RTG, RTG_n+5)
    set_servo_deg(RKN, RKN_close+20)
    set_servo_deg(RAK, RAK_close-40)
    set_servo_deg(RTE, RTE_n)
    time.sleep(wait_time)

    #右足下へ下す
    set_servo_deg(RTG, RTG_n+5)
    set_servo_deg(RKN, RKN_close+20)
    set_servo_deg(RAK, RAK_open+5)
    set_servo_deg(RTE, RTE_open-15)
    time.sleep(wait_time)


    #右足後に出す
    set_servo_deg(RTG, RTG_n)
    set_servo_deg(RKN, RKN_open-40)
    set_servo_deg(RAK, RAK_open+5)
    set_servo_deg(RTE, RTE_close+30)
    time.sleep(wait_time)

def sv_walk_cw(wait_time):
    #左足上に上げる
    set_servo_deg(LTG, LTG_n)
    set_servo_deg(LKN, LKN_open+40)
    set_servo_deg(LAK, LAK_close+35)
    set_servo_deg(LTE, LTE_n)
    time.sleep(wait_time)

    #左足前へ出す
    set_servo_deg(LTG, LTG_n-8)
    set_servo_deg(LKN, LKN_close-20)
    set_servo_deg(LAK, LAK_close+35)
    set_servo_deg(LTE, LTE_n)
    time.sleep(wait_time)

    #左足下へ下す
    set_servo_deg(LTG, LTG_n-8)
    set_servo_deg(LKN, LKN_close-20)
    set_servo_deg(LAK, LAK_open-10)
    set_servo_deg(LTE, LTE_open+10)
    time.sleep(wait_time)

    #左足後に出す
    set_servo_deg(LTG, LTG_n)
    set_servo_deg(LKN, LKN_open+35)
    set_servo_deg(LAK, LAK_open-10)
    set_servo_deg(LTE, LTE_close-30)
    time.sleep(wait_time)



def move_test():
    sv_spuat()
    time.sleep(1)
#    sv_init()
#    time.sleep(1)
    sv_stretch()
    time.sleep(1)
#    sv_init()
#    time.sleep(1)


def sv_max_min():
#    set_servo_deg(RTG, RTG_open)
#    set_servo_deg(LTG, LTG_open)
#    set_servo_deg(RKN, RKN_open)
#    set_servo_deg(LKN, LKN_open)
    set_servo_deg(RAK, RAK_open)
    set_servo_deg(LAK, LAK_open)
#    set_servo_deg(RTE, RTE_open)
#    set_servo_deg(LTE, LTE_open)

    time.sleep(1)
#    set_servo_deg(RTG, RTG_close)
#    set_servo_deg(LTG, LTG_close)
#    set_servo_deg(RKN, RKN_close)
#    set_servo_deg(LKN, LKN_close)
    set_servo_deg(RAK, RAK_close)
    set_servo_deg(LAK, LAK_close)
#    set_servo_deg(RTE, RTE_close)
#    set_servo_deg(LTE, LTE_close)

    time.sleep(1)

def main():
    global speed
    global walk_st

    print("main start")
    sv_init()
    time.sleep(2)
    speed = high
    print("Inisialize OK")
    while True:
       sv_walk_fw(0.2)
       sv_walk_fw(0.2)
       sv_walk_fw(0.2)
       sv_walk_fw(0.2)
       time.sleep(1)
       sv_walk_cw(0.2)
       sv_walk_cw(0.2)
       sv_walk_cw(0.2)
       sv_walk_cw(0.2)
       time.sleep(1)
       #move_test()
       #sv_max_min()
       pass


if __name__ == '__main__':
    main()
